/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Spatial SDK, Version 7.0                   */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* External Position and Velocity */
public class ANPacket44
{
	public final static int LENGTH = 60;

	/* radians, radians, metres */
	public double[] position;
	/* metres per second */
	public float[] velocity;
	/* metres */
	public float[] positionStandardDeviation;
	/* metres per second */
	public float[] velocityStandardDeviation;
	
	public ANPacket44()
	{
		position = new double[3];
		velocity = new float[3];
		positionStandardDeviation = new float[3];
		velocityStandardDeviation = new float[3];
	}

	public ANPacket44(ANPacket packet)
	{
		position = new double[3];
		position[0] = TypeConversion.bytesToDouble(packet.data, 0);
		position[1] = TypeConversion.bytesToDouble(packet.data, 8);
		position[2] = TypeConversion.bytesToDouble(packet.data, 16);
		
		velocity = new float[3];
		velocity[0] = TypeConversion.bytesToFloat(packet.data, 24);
		velocity[1] = TypeConversion.bytesToFloat(packet.data, 28);
		velocity[2] = TypeConversion.bytesToFloat(packet.data, 32);
		
		positionStandardDeviation = new float[3];
		positionStandardDeviation[0] = TypeConversion.bytesToFloat(packet.data, 36);
		positionStandardDeviation[1] = TypeConversion.bytesToFloat(packet.data, 40);
		positionStandardDeviation[2] = TypeConversion.bytesToFloat(packet.data, 44);
		
		velocityStandardDeviation = new float[3];
		velocityStandardDeviation[0] = TypeConversion.bytesToFloat(packet.data, 48);
		velocityStandardDeviation[1] = TypeConversion.bytesToFloat(packet.data, 52);
		velocityStandardDeviation[2] = TypeConversion.bytesToFloat(packet.data, 56);
	}
	
	public ANPacket toANPacket()
	{	
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_EXTERNAL_POSITION_AND_VELOCITY);
		TypeConversion.doubleToBytes(packet.data, 0, position[0]);
		TypeConversion.doubleToBytes(packet.data, 8, position[1]);
		TypeConversion.doubleToBytes(packet.data, 16, position[2]);		
		TypeConversion.floatToBytes(packet.data, 24, velocity[0]);
		TypeConversion.floatToBytes(packet.data, 28, velocity[1]);
		TypeConversion.floatToBytes(packet.data, 32, velocity[2]);
		TypeConversion.floatToBytes(packet.data, 36, positionStandardDeviation[0]);
		TypeConversion.floatToBytes(packet.data, 40, positionStandardDeviation[1]);
		TypeConversion.floatToBytes(packet.data, 44, positionStandardDeviation[2]);
		TypeConversion.floatToBytes(packet.data, 48, velocityStandardDeviation[0]);
		TypeConversion.floatToBytes(packet.data, 52, velocityStandardDeviation[1]);
		TypeConversion.floatToBytes(packet.data, 56, velocityStandardDeviation[2]);			
		return packet;
	}	
}
